(use-translation _ "alterator-xkb")

(define (on-load-error reason) '())

(define (convert-to-pair name x)
  (cons (sure-symbol (woo-extract-name name x))
	(woo-get-option x 'description)))

(define (get-list name)
  (woo-catch
    (lambda()
      (sort
	(map 
	  (lambda (x)
	    (convert-to-pair name x))
	  (woo-list name))
	(lambda (x y)
	  (string<? (cdr x) (cdr y)))))
    on-load-error))


(define (get-current)
  (woo-first-command (woo-read "/xkb/current")))


(define *xkb-models* (get-list "/xkb/models"))
(define *xkb-layouts* (get-list "/xkb/layouts"))
(define *xkb-option-groups* (get-list "/xkb/option_groups"))
(define *xkb-switches* (get-list "/xkb/option_groups/grp"))

(define *xkb-current* (woo-catch get-current
				 on-load-error))

(define (current->list name)
  (let ((value (cond-cdr (command-arg-ref *xkb-current* name))))
    (or (and value
	     (map sure-symbol (string-splitting value
						#\,
						#t)))
	'())))


;extract current switch settings
(define *current-options* (current->list 'options))
(define *current-model* (cond-cdr (command-arg-ref *xkb-current* 'model)))

(define (default-variants)
  (list
    (cons (string->symbol "")
	  (_ "default"))))

(define *selected-variants* (default-variants))

;find first of "grp:" options
(define *current-switch* (list-index (lambda (x) (member (car x) *current-options*)) ;x one of the options
				     *xkb-switches*))

;remove founded switch from main list of the options
(if *current-switch*
	  (set! *current-options* (delq 
				    (car (list-ref *xkb-switches* *current-switch*))
				    *current-options*)))

(define (not-current-layout? x) (not (member (car x) *current-layouts*)))

;current layouts
(define *current-layouts*  (current->list 'layout))
;other available layouts
(define *avail-layouts* (map car (filter not-current-layout?
					     *xkb-layouts*)))
;current variants
(define (make-variants lst len)
  (if (null? lst)
	(map sure-symbol (make-list len ""))
	lst))
(define *current-variants* (make-variants (current->list 'variant) (length *current-layouts*)))


(define (get-layout-name x) (cdr (assq x *xkb-layouts*)))


(define (list-remove lst num)
    (append (list-head lst num) (list-tail lst (+ num 1))))

(define (list-transpose! lst num)
  (let* ((tail (drop lst num))
	 (elem1 (car tail))
	 (elem2 (cadr tail)))
	
    (set-car! tail elem2)
    (set-car! (cdr tail) elem1)))

(on-acc-leave (if (xkb-is-dirty?) 'need-confirm))

((global 'disable-next))
(define (xkb-mark-as-dirty)
  (next-button enabled (> (current-layouts count) 0)))
(define (xkb-clear-dirty)
  (next-button enabled #f))
(define (xkb-is-dirty?)
  (next-button enabled))

(define (get-current-options)
  (append (list (car (list-ref *xkb-switches* (avail-switches current))))
	  *current-options*))

(on-apply
  (woo-write "/xkb/current"
	     'layout (string-join (map sure-string *current-layouts*) ",")
	     'variant (string-join (map sure-string *current-variants*) ",")
	     'model (sure-string (car (list-ref *xkb-models* (avail-models current))))
	     'options (string-join (map sure-string (get-current-options)) ","))
  (woo-write "/xkb/reload")
  (xkb-clear-dirty))


(define (remove-layout)
  (let ((number (current-layouts current)))
    (and (>= number 0)
	 (let ((layout (list-ref *current-layouts* number)))
	   ;reset current layouts and appropriate listbox
	   (current-layouts remove number)
	   (set! *current-layouts* (list-remove *current-layouts* number))
	   (set! *current-variants* (list-remove *current-variants* number))

	   ;reset available layouts and appropriate listbox
	   (set! *avail-layouts* (sort
				   (append *avail-layouts* (list layout))
				   (lambda (x y)
				     (string<? (get-layout-name x)
					       (get-layout-name y)))))
	   (avail-layouts items (map get-layout-name *avail-layouts*))
	   (xkb-mark-as-dirty)))))

(define (add-layout)
  (let ((number (avail-layouts current)))
    (and (>= number 0)
	 (let ((layout (list-ref *avail-layouts* number)))
	   ;reset avail layouts
	   (avail-layouts remove number)
	   (set! *avail-layouts* (list-remove *avail-layouts* number))

	   ;reset current layouts
	   (set! *current-layouts* (append *current-layouts* (list layout)))
	   (set! *current-variants* (append *current-variants* (list (string->symbol ""))))
	   (current-layouts append-item (get-layout-name layout))
	   (xkb-mark-as-dirty)))))

(define (up-layout)
  (let ((number (- (current-layouts current) 1)))
    (and (>= number 0)
	 (begin
	   (list-transpose! *current-layouts* number)
	   (list-transpose! *current-variants* number)
	   (current-layouts items (map get-layout-name *current-layouts*))
	   (current-layouts current number)
	   (xkb-mark-as-dirty)))))

(define (down-layout)
  (let ((number (current-layouts current)))
    (and (>= number 0)
	 (< number (- (length *current-layouts*) 1))
	 (begin
	   (list-transpose! *current-layouts* number)
	   (list-transpose! *current-variants* number)
	   (current-layouts items (map get-layout-name *current-layouts*))
	   (current-layouts current (+ number 1))
	   (xkb-mark-as-dirty)))))

(define (up-down-buttons)
  (hbox
    (button (_ "Up")
	    sizepolicy '(expanding . fixed)
	    (on-click (up-layout)))
    (button (_ "Down")
	    sizepolicy '(expanding . fixed)
	    (on-click (down-layout)))))

(define (add-remove-buttons)
  (hbox
    (button (_ "Add")
	    sizepolicy '(expanding . fixed)
	    (on-click (add-layout)))
    (button (_ "Remove")
	    sizepolicy '(expanding . fixed)
	    (on-click (remove-layout)))))

(define (current-layout-name)
  (sure-string
    (list-ref *current-layouts* (current-layouts current))))

(define (variant-index variant)
  (or
    (and (pair? *selected-variants*)
	 (list-index (lambda(x)
		       (eq? (car x) variant))
		     *selected-variants*))
    0))

(define (update-selected-variants)
  (set! *selected-variants*
    (woo-catch (lambda()
		 (append (default-variants)
			 (get-list (string-append
				     "/xkb/layouts/"
				     (current-layout-name)
				     "/variants"))))
	       (lambda(reason)
		 (default-variants)))))

(define (update-variants)
  (if (>= (current-layouts current) 0)
       (begin (update-selected-variants)
	      (avail-variants
		items
		(map cdr *selected-variants*))
	      (avail-variants current (variant-index (list-ref *current-variants*
							       (current-layouts current)))))
       (avail-variants remove 'all)))

(define (set-variants)
  (let ((number (current-layouts current))
	(variant (avail-variants current)))
    (and (>= number 0)
	 (begin
	   (set-car! (drop *current-variants* number)
		     (car (list-ref *selected-variants* variant)))
	   (xkb-mark-as-dirty)))))

(define (model-index model)
  (and model
       (list-index (lambda (x)
		     (eq? (sure-symbol model) (car x)))
		   *xkb-models*)))

(tabbox 
  (tab 'layoutbox
       (_ "Layouts")
       (vbox
	 spacing 1
	 margin 1
	 (groupbox (_ "Model")
		   (id 'avail-models (combobox 
				       (items (map cdr *xkb-models*))
				       current (or (model-index *current-model*)
						   (model-index 'pc105)
						   0)
				       (on-change (xkb-mark-as-dirty)))))
	 (groupbox (_ "Layouts")
		   (vbox
		     spacing 1
		     (vbox
		       (label (_ "Available Layouts:"))
		       (id 'avail-layouts (listbox
					    (relative-height 20 page)
					    (items (map get-layout-name *avail-layouts*))))
		       (add-remove-buttons)
		       (id 'current-layouts (listbox
					      (items (map get-layout-name *current-layouts*))
					      (on-select
						(update-variants))))
					      
		       (up-down-buttons))
		     (hbox
		       (label (_ "Variant:"))
		       (id 'avail-variants (combobox
					     (on-change (set-variants)))))))
	 (groupbox (or (cond-cdr (assq 'grp *xkb-option-groups*)) "Switch")
		   (id 'avail-switches
		       (combobox (items (map cdr *xkb-switches*))
				 (if *current-switch* (current *current-switch*))
				 (on-change (xkb-mark-as-dirty))))))))
;  (tab 'modelbox
;       (_ "Advanced")
;       (vbox
;	 (label ""))))


